package com.starbuds.server.api.notify.v1.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.format.FastDateFormat;
import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.ReUtil;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.notify.v1.entity.TencentNotifyResult;
import com.starbuds.server.api.notify.v1.facade.ImFacade;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.message.ImService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import java.util.Optional;
import java.util.regex.Pattern;

/**
 * IM 消息通知实现
 *
 * @author Lay
 * @date 2019/12/24
 */
@Path("/v1/im")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ImFacadeImpl extends BaseFacade implements ImFacade {

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    CacheService cacheService;

    private static final Pattern pattern = Pattern.compile("\\\\n\\s+");

    /**
     * 腾讯 IM 消息通知
     *
     * @param sdkAppId
     * @param callbackCommand
     * @param clientIp
     * @param optPlatform
     * @param request
     * @return
     */
    @POST
    @Path("tencent")
    @PermitAll
    @Override
    public TencentNotifyResult tencent(@QueryParam("SdkAppid") Long sdkAppId,
                                       @QueryParam("CallbackCommand") String callbackCommand,
                                       @QueryParam("ClientIP") String clientIp,
                                       @QueryParam("OptPlatform") String optPlatform,
                                       @Context HttpServletRequest request) {
        // 判断 SdkAppId
        if (!imService.checkSdkAppId(sdkAppId)) {
            return buildError("SdkAppId 不匹配");
        }

        // 处理
        try {
            String body = ReUtil.replaceAll(IOUtils.toString(request.getInputStream(), CharsetUtil.UTF_8), pattern, "");
            log.info("收到腾讯IM通知 {}", body);
            if (imService.handleTencentImCallback(callbackCommand, clientIp, optPlatform, body)) {
                return buildSuccess();
            }
        } catch (Exception e) {
            log.error("腾讯 IM 回调处理失败", e);
        }

        return buildError("");
    }

    /**
     * 聊天室状态同步
     *
     * @param signTimestamp
     * @param nonce
     * @param signature
     * @param request
     * @return
     */
    @POST
    @Path("roomStatus")
    @Consumes("application/json; charset=utf-8")
    @PermitAll
    @Override
    public String roomStatus(@QueryParam("signTimestamp") String signTimestamp,
                             @QueryParam("nonce") String nonce,
                             @QueryParam("signature") String signature,
                             @Context HttpServletRequest request) {
        if (StringUtils.isNotBlank(signTimestamp) && StringUtils.isNotBlank(nonce) && StringUtils.isNotBlank(signature)) {
            // 验签
            if (imService.checkRongCloudSign(signTimestamp, nonce, signature)) {
                // 处理
                try {
                    String body = ReUtil.replaceAll(IOUtils.toString(request.getInputStream(), CharsetUtil.UTF_8), pattern, "");
                    log.info("收到融云通知 {}", body);
                    imService.handleRongCloudRoomStatusNotify(body);
                } catch (Exception e) {
                    log.error("融云房间状态同步通知处理失败", e);
                }
            }
        }

        return null;
    }

    /**
     * 在线状态同步
     *
     * @param signTimestamp
     * @param nonce
     * @param signature
     * @param request
     * @return
     */
    @POST
    @Path("onlineStatus")
    @Consumes("application/json; charset=utf-8")
    @PermitAll
    @Override
    public String onlineStatus(@QueryParam("signTimestamp") String signTimestamp,
                               @QueryParam("nonce") String nonce,
                               @QueryParam("signature") String signature,
                               @Context HttpServletRequest request) {
        if (StringUtils.isNotBlank(signTimestamp) && StringUtils.isNotBlank(nonce) && StringUtils.isNotBlank(signature)) {
            // 验签
            if (imService.checkRongCloudSign(signTimestamp, nonce, signature)) {
                // 处理
                try {
                    String body = ReUtil.replaceAll(IOUtils.toString(request.getInputStream(), CharsetUtil.UTF_8), pattern, "");
                    log.info("收到融云通知 {}", body);
                    imService.handleRongCloudUserStatusNotify(body);
                } catch (Exception e) {
                    log.error("融云用户在线状态同步通知处理失败", e);
                }
            }
        }

        return "success";
    }

    /**
     * 融云消息回调
     *
     * @param signTimestamp
     * @param nonce
     * @param signature
     * @return
     */
    @POST
    @Path("rongCloudMsg")
    @PermitAll
    @Override
    public String rongCloudMsg(@QueryParam("signTimestamp") String signTimestamp,
                               @QueryParam("nonce") String nonce,
                               @QueryParam("signature") String signature,
                               @FormParam("channelType") String channelType,
                               @FormParam("objectName") String objectName,
                               @Context HttpServletRequest request
    ) {
//        boolean filterParam = StringUtils.isNotBlank(signTimestamp) && StringUtils.isNotBlank(nonce) && StringUtils.isNotBlank(signature);
//        if (!filterParam) {
//            log.info("--rongCloudMsg接收参数失败：signTimestamp："+signTimestamp + "  nonce："+nonce + "  signature："+signature);
//            return "error";
//        }
//
//        if (!imService.checkRongCloudSign(signTimestamp, nonce, signature)) {
//            log.info("--rongCloudMsg验签失败：signTimestamp："+signTimestamp + "  nonce："+nonce + "  signature："+signature);
//            return "error";
//        }
//
//        channelType = Optional.ofNullable(channelType).orElse("");
//        if (!channelType.equals("TEMPGROUP")) {
//            return "success";
//        }
//
//        final String rcTxtMsg = "RC:TxtMsg";
//        final String rcCmdMsg = "RC:CmdMsg";
//        final String msgTypeText    = "live:msg:text";
//        final String msgTypeFacial  = "rtc:facial:expression";
//
//        try {
//            String content = Optional.ofNullable(request.getParameterMap()).map(m->m.get("content")[0]).orElse("{}");
//            JSONObject jsonObject = JSONObject.parseObject(content);
//            JSONObject jsonData = null;
//
//            if (rcTxtMsg.equals(objectName)) {
//                jsonData = JSONObject.parseObject(jsonObject.getString("extra"));
//            }
//            else if (rcCmdMsg.equals(objectName)) {
//                jsonData = JSONObject.parseObject(jsonObject.getString("data"));
//            }
//            else {
//                return "filterRcMSG";
//            }
//
//            JSONObject msgObject = jsonData.getJSONObject("msg");
//            String msgType = Optional.ofNullable(msgObject).map(json->json.getString("type")).orElse("");
//
//            Long senderID = 0L;
//            Long roomID = 0L;
//
//            if (msgTypeText.equals(msgType)) {
//                senderID = jsonData.getLong("senderId");
//                roomID = jsonData.getLong("targetId");
//            }
//            else if (msgTypeFacial.equals(msgType)) {
//                senderID = jsonData.getLong("senderId");
//                roomID = jsonData.getLong("groupId");
//            }
//            else {
//                log.info("-- rongCloudMsg filter:"+msgType);
//                return "filter";
//            }
//
//            String strToday = FastDateFormat.getInstance("dd").format(new DateTime());
//            String roomsKey = RedisCacheKeys.getLiveBarrageDayKey(strToday,0L);
//            cacheService.zIncrementScore(roomsKey,roomID + "",1);
//
//            String key = RedisCacheKeys.getLiveBarrageDayKey(strToday, roomID);
//            cacheService.zIncrementScore(key, senderID + "", 1);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }

        return "";
    }

    private static TencentNotifyResult buildError(String reason) {
        return TencentNotifyResult.builder().errorCode(BoolIntVal.True).actionStatus("FAIL").errorInfo(reason).build();
    }

    private static TencentNotifyResult buildSuccess() {
        return TencentNotifyResult.builder().errorCode(BoolIntVal.False).actionStatus("OK").errorInfo("").build();
    }
}
